package sun.security.provider.certpath;

import sun.misc.HexDumpEncoder;
import java.util.Arrays;
import sun.security.util.DerOutputStream;
import java.math.BigInteger;
import sun.security.util.DerInputStream;
import sun.security.util.DerValue;
import java.security.NoSuchAlgorithmException;
import java.security.MessageDigest;
import java.security.PublicKey;
import javax.security.auth.x500.X500Principal;
import java.io.IOException;
import java.security.cert.X509Certificate;
import sun.security.x509.SerialNumber;
import sun.security.x509.AlgorithmId;

public class CertId
{
    private static final boolean debug = false;
    private static final AlgorithmId SHA1_ALGID;
    private final AlgorithmId hashAlgId;
    private final byte[] issuerNameHash;
    private final byte[] issuerKeyHash;
    private final SerialNumber certSerialNumber;
    private int myhash;
    
    public CertId(X509Certificate x509Certificate, SerialNumber serialNumber) throws IOException {
        this(x509Certificate.getSubjectX500Principal(), x509Certificate.getPublicKey(), serialNumber);
    }
    
    public CertId(X500Principal x500Principal, PublicKey publicKey, SerialNumber certSerialNumber) throws IOException {
        this.myhash = -1;
        MessageDigest instance;
        try {
            instance = MessageDigest.getInstance("SHA1");
        }
        catch (NoSuchAlgorithmException cause) {
            throw new IOException("Unable to create CertId", cause);
        }
        this.hashAlgId = CertId.SHA1_ALGID;
        instance.update(x500Principal.getEncoded());
        this.issuerNameHash = instance.digest();
        DerValue derValue = new DerValue(publicKey.getEncoded());
        instance.update((new DerValue[] { derValue.data.getDerValue(), derValue.data.getDerValue() })[1].getBitString());
        this.issuerKeyHash = instance.digest();
        this.certSerialNumber = certSerialNumber;
    }
    
    public CertId(DerInputStream in) throws IOException {
        this.myhash = -1;
        this.hashAlgId = AlgorithmId.parse(in.getDerValue());
        this.issuerNameHash = in.getOctetString();
        this.issuerKeyHash = in.getOctetString();
        this.certSerialNumber = new SerialNumber(in);
    }
    
    public AlgorithmId getHashAlgorithm() {
        return this.hashAlgId;
    }
    
    public byte[] getIssuerNameHash() {
        return this.issuerNameHash;
    }
    
    public byte[] getIssuerKeyHash() {
        return this.issuerKeyHash;
    }
    
    public BigInteger getSerialNumber() {
        return this.certSerialNumber.getNumber();
    }
    
    public void encode(DerOutputStream derOutputStream) throws IOException {
        DerOutputStream derOutputStream2 = new DerOutputStream();
        this.hashAlgId.encode(derOutputStream2);
        derOutputStream2.putOctetString(this.issuerNameHash);
        derOutputStream2.putOctetString(this.issuerKeyHash);
        this.certSerialNumber.encode(derOutputStream2);
        derOutputStream.write((byte)48, derOutputStream2);
    }
    
    @Override
    public int hashCode() {
        if (this.myhash == -1) {
            this.myhash = this.hashAlgId.hashCode();
            for (int i = 0; i < this.issuerNameHash.length; ++i) {
                this.myhash += this.issuerNameHash[i] * i;
            }
            for (int j = 0; j < this.issuerKeyHash.length; ++j) {
                this.myhash += this.issuerKeyHash[j] * j;
            }
            this.myhash += this.certSerialNumber.getNumber().hashCode();
        }
        return this.myhash;
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || !(o instanceof CertId)) {
            return false;
        }
        CertId certId = (CertId)o;
        return this.hashAlgId.equals(certId.getHashAlgorithm()) && Arrays.equals(this.issuerNameHash, certId.getIssuerNameHash()) && Arrays.equals(this.issuerKeyHash, certId.getIssuerKeyHash()) && this.certSerialNumber.getNumber().equals(certId.getSerialNumber());
    }
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("CertId \n");
        sb.append("Algorithm: " + this.hashAlgId.toString() + "\n");
        sb.append("issuerNameHash \n");
        HexDumpEncoder hexDumpEncoder = new HexDumpEncoder();
        sb.append(hexDumpEncoder.encode(this.issuerNameHash));
        sb.append("\nissuerKeyHash: \n");
        sb.append(hexDumpEncoder.encode(this.issuerKeyHash));
        sb.append("\n" + this.certSerialNumber.toString());
        return sb.toString();
    }
    
    static {
        SHA1_ALGID = new AlgorithmId(AlgorithmId.SHA_oid);
    }
}
